from SETMOK_API.util.Mysql import MySql
from SETMOK_API.SETMOKE_API import SETMOKE_API

class Scheduler:
    def __init__(self,host, database_user, database_password, database_name):

       self.db= MySql(host, database_user, database_password, database_name)

    def read_and_update_database(self):
        result_set=self.db.read_keyword_from_db()
        for row in result_set:
         keyword=str(row[0])
         keyword_id=str(row[1])
         data_fetch=SETMOKE_API(keyword,"/home/rehab/PycharmProjects/SETMOK_API/conf/config.ini")
         mentionlist=data_fetch.get_data()
         self.db.update_data_base(mentionlist, keyword_id)





from SETMOK_API.util.ConfigurationParser import ConfigurationParser
from SETMOK_API.src.GooglePlusDataRetreivel import GooglePlus
from SETMOK_API.src.TwitterDataRetrievel import S_Twitter
import json
import sys
from SETMOK_API.util.Mysql import MySql
from SETMOK_API.util.SchedulerScript import Scheduler

class SETMOKE:

    def __init__(self, keyword=None, configpath=None,limit=2,source=None):
        self.set_keyword(keyword)
        self.set_config_path(configpath)
        self.set_source(source)
        self.set_limit(limit)

    def set_limit(self, limit):
        self.limit = limit

    def set_keyword(self,keyword):
        self.keyword = keyword
        if self.get_keyword()== None:

            sys.exit("Please provide a keyword to search \n SETMOKE_API(<keyword>,<config_path>,<source>,<limit>)")

    def set_config_path(self, config_path):
        self.config_path = config_path
        if self.get_config_path()==None :
            sys.exit("config_path is empty.Please provide config file path \n SETMOKE_API(<keyword>,<config_path>,<source>,<limit>)")

    def set_source(self, source):

        self.source = source

    def get_limit(self):
        return self.limit

    def get_keyword(self):
        return self.keyword

    def get_config_path(self):
        return self.config_path

    def get_source(self):
        return self.source

    def save_to_file(self,list,source):
        length = len(list)
        writer = open(source, 'w')
        writer.write("[\n")
        # jsonArray=[]
        count = 1
        for items in list:

            resharers = items.get_resharer()
            if not resharers:
                data_resharer = '[]'
            else:
                resharers = items.get_resharer()
                data_resharer = []

                for res in resharers:
                    dict = {}
                    dict["display_name"] = res.get_display_name()
                    dict["display_picture"] = res.get_display_picture()
                    dict["time"] = res.get_time()
                    data_resharer.append(dict)

            my_json_string = json.dumps({'text': items.get_text(), 'display_name': items.get_display_name(),
                                         'time': items.get_time(),
                                         'reshare_count': items.get_reshare_count(),
                                         'status_id': items.get_status_id(),
                                         'source': items.get_source(),
                                         'resharers': data_resharer})
            if count < length:
                writer.write(my_json_string + ",")
            else:
                writer.write(my_json_string)
            count = count + 1

        writer.write("]")
    def get_googleplus_data(self):
        self.set_source("googlePlus")
        obj = ConfigurationParser(self.get_config_path())
        gp_api_credentials = obj.get("googlePlus_credentials")
        api_key = gp_api_credentials["api_key"]
        gp = GooglePlus(api_key)
        googlePlus_mentionlist = gp.get_googleplus_data(self.keyword, self.limit)
        print(googlePlus_mentionlist)
        return googlePlus_mentionlist
       # self.save_to_file(googlePlus_mentionlist, self.get_source())

    def get_Twitter_data(self):
        self.set_source("Twitter")
        obj = ConfigurationParser(self.get_config_path())
        twitter_api_credentials = obj.get("twitter_credentials")
        consumer_key = twitter_api_credentials["consumer_key"]
        consumer_secret = twitter_api_credentials["consumer_secret"]
        oauth_token = twitter_api_credentials["oauth_token"]
        oauth_token_secret = twitter_api_credentials["oauth_token_secret"]
        twitter = S_Twitter(consumer_key, consumer_secret, oauth_token, oauth_token_secret)
        twitter_mentionlist = twitter.get_twitter_data(self.keyword, self.limit)
        return twitter_mentionlist
        #self.save_to_file(twitter_mentionlist, self.get_source())

    def get_data(self):
        if self.get_source()=='googlePlus':
            return self.get_googleplus_data()
        elif self.get_source()=='twitter':
            return self.get_Twitter_data()
        else :
            google_data=self.get_googleplus_data()
            twitter_data=self.get_Twitter_data()
            data=google_data+twitter_data
            return data

    def add_to_database(self, mentionlist, host,database_user,database_pwd,database_name):
        db = MySql(host, database_user, database_pwd, database_name)
        db.add_kwd(self.get_keyword(), mentionlist)

    def read_and_update_database(self,host,database_user,database_pwd,database_name):
        schedule=Scheduler(host,database_user,database_pwd,database_name)
        schedule.read_and_update_database()




import pymysql
from datetime import datetime
import dateparser


class MySql:
    db=None
    cursor=None
    def __init__(self,host,user,password,db):

     self.db = pymysql.connect(host=host, user=user, passwd=password,db=db)

    def add_kwd(self, keyword, result, required_keyword=None, optional_keyword=None, excluded_keyword=None):
        self.cursor = self.db.cursor()
        self.cursor.execute("""INSERT INTO Periodic_keyword(Text, Optional, Required, Excluded) VALUES(%s,%s,%s,%s)""", (keyword, optional_keyword, required_keyword, excluded_keyword))
        self.cursor.execute("SELECT LAST_INSERT_ID(); ")
        keywordID=self.cursor.fetchone()
        for items in result:
            display_name = items.get_user().get_display_name()
            display_picture = items.get_user().get_display_picture()
            follower_count = items.get_user().get_follower_count()
            following_count = items.get_user().get_following_count()
            created_at = items.get_user().get_time()
            total_likes = items.get_user().get_total_likes()
            total_post = items.get_user().get_total_post()
            PostReshareCount = None
            userID = items.get_user().get_user_id()
            self.cursor.execute(
                """INSERT INTO Periodic_postuser (DisplayImage, DisplayName, FollowerCount,FollowingCount,TotalPosts,PostReshareCount,TotalLikes, UserID) VALUES (%s,%s,%s,%s,%s,%s,%s,%s)""",
                (display_picture, display_name, follower_count, following_count, total_post, PostReshareCount,
                 total_likes, userID))
            self.db.commit()
            self.cursor.execute("SELECT LAST_INSERT_ID(); ")
            postUserID = self.cursor.fetchone()
            post_content=items.get_text()
            time= items.get_user().get_time()
            user_id=items.get_user().get_user_id()
            source=items.get_source()
            resharer_count=items.get_reshare_count()
            post_id=items.get_status_id()


            self.cursor.execute("""Insert into Periodic_post(Keyword_id, PostUser_id, statusID,Content,CreatedAt, Source, ResharerCount) VALUES (%s,%s,%s,%s,%s,%s,%s)""",(keywordID,postUserID,post_id,post_content,time,source,resharer_count))
            self.db.commit()

            self.cursor.execute("SELECT LAST_INSERT_ID(); ")
            self.postID = self.cursor.fetchone()
            if resharer_count!='0':
              for user in items.get_resharer():
                  display_name = user.get_display_name()
                  display_picture = user.get_display_picture()
                  follower_count = user.get_follower_count()
                  following_count = user.get_following_count()
                  created_at = user.get_time()
                  total_likes = user.get_total_likes()
                  total_post = user.get_total_post()
                  PostReshareCount = None
                  userID = user.get_user_id()
                  self.cursor.execute(
                      """INSERT INTO Periodic_postuser (DisplayImage, DisplayName, FollowerCount,FollowingCount,TotalPosts,PostReshareCount,TotalLikes, UserID) VALUES (%s,%s,%s,%s,%s,%s,%s,%s)""",
                      (display_picture, display_name, follower_count, following_count, total_post, PostReshareCount,
                       total_likes, userID))
                  self.db.commit()
                  self.cursor.execute("SELECT LAST_INSERT_ID(); ")
                  postUserID = self.cursor.fetchone()
                  PostID=self.postID
                  self.cursor.execute(
                      """INSERT INTO Periodic_resharer (Post_id, PostUser_id) VALUES (%s,%s)""",
                      (PostID, postUserID))
                  self.db.commit()

        self.db.commit()
        #self.db.close()


    def add_to_Keyword(self, keyword, result, required_keyword=None, optional_keyword=None, excluded_keyword=None):
        self.cursor = self.db.cursor()
        self.cursor.execute("""INSERT INTO Keyword(Text, Optional, Required, Excluded) VALUES(%s,%s,%s,%s)""",
                            (keyword, optional_keyword, required_keyword, excluded_keyword))
        self.cursor.execute("SELECT LAST_INSERT_ID(); ")
        keywordID = self.cursor.fetchone()
    def read_keyword_from_db(self):
         self.cursor = self.db.cursor()
         self.cursor.execute("""SELECT Text, id from Periodic_keyword""")
         result_set = self.cursor.fetchall()

         return  result_set
    def update_data_base(self, result, keyword_id):
        for items in result:
            display_name = items.get_user().get_display_name()
            display_picture = items.get_user().get_display_picture()
            follower_count = items.get_user().get_follower_count()
            following_count = items.get_user().get_following_count()
            created_at = items.get_user().get_time()
            total_likes = items.get_user().get_total_likes()
            total_post = items.get_user().get_total_post()
            PostReshareCount = None
            userID = items.get_user().get_user_id()
            self.cursor.execute(
                """INSERT INTO Periodic_postuser (DisplayImage, DisplayName, FollowerCount,FollowingCount,TotalPosts,PostReshareCount,TotalLikes, UserID) VALUES (%s,%s,%s,%s,%s,%s,%s,%s)""",
                (display_picture, display_name, follower_count, following_count, total_post, PostReshareCount,
                 total_likes, userID))
            self.db.commit()
            self.cursor.execute("SELECT LAST_INSERT_ID(); ")
            postUserID = self.cursor.fetchone()
            post_content = items.get_text()
            time = items.get_user().get_time()
            user_id = items.get_user().get_user_id()
            source = items.get_source()
            resharer_count = items.get_reshare_count()
            post_id = items.get_status_id()

            self.cursor.execute(
                """Insert into Periodic_post(Keyword_id, PostUser_id, statusID,Content,CreatedAt, Source, ResharerCount) VALUES (%s,%s,%s,%s,%s,%s,%s)""",
                (keyword_id, postUserID, post_id, post_content, time, source, resharer_count))
            self.db.commit()

            self.cursor.execute("SELECT LAST_INSERT_ID(); ")
            self.postID = self.cursor.fetchone()
            if resharer_count != '0':
                for user in items.get_resharer():
                    display_name = user.get_display_name()
                    display_picture = user.get_display_picture()
                    follower_count = user.get_follower_count()
                    following_count = user.get_following_count()
                    created_at = user.get_time()
                    total_likes = user.get_total_likes()
                    total_post = user.get_total_post()
                    PostReshareCount = None
                    userID = user.get_user_id()
                    self.cursor.execute(
                        """INSERT INTO Periodic_postuser (DisplayImage, DisplayName, FollowerCount,FollowingCount,TotalPosts,PostReshareCount,TotalLikes, UserID) VALUES (%s,%s,%s,%s,%s,%s,%s,%s)""",
                        (display_picture, display_name, follower_count, following_count, total_post, PostReshareCount,
                         total_likes, userID))
                    self.db.commit()
                    self.cursor.execute("SELECT LAST_INSERT_ID(); ")
                    postUserID = self.cursor.fetchone()
                    PostID = self.postID
                    self.cursor.execute(
                        """INSERT INTO Periodic_resharer (Post_id, PostUser_id) VALUES (%s,%s)""",
                        (PostID, postUserID))
                    self.db.commit()

        self.db.commit()
        # self.db.close()



class User:
    def __init__(self):

        self.display_name = None
        self.display_picture = None
        self.follower_count=None
        self.following_count=None
        self.total_likes=None
        self.total_post= None
        self.user_id=None
        self.time=None

    def set_display_name(self, display_name):
        self.display_name = display_name
    def set_display_picture(self, display_picture):
        self.display_picture = display_picture
    def set_time(self, time):
        self.time = time
    def set_follower_count(self, follower_count):
        self.follower_count=follower_count
    def set_following_count (self,following_count):
        self.following_count=following_count
    def set_total_likes(self,total_likes):
        self.total_likes=total_likes
    def set_total_post(self,total_post):
        self.total_post=total_post
    def set_user_id(self,user_id):
        self.user_id=user_id
    def set_time(self,time):
        self.time=time

    def get_display_name(self):
        return self.display_name
    def get_display_picture(self):
        return self.display_picture
    def get_follower_count(self):
        return self.follower_count
    def get_following_count(self) :
        return self.following_count
    def get_total_likes(self):
        return self.total_likes
    def get_total_post(self):
        return self.total_post
    def get_user_id(self):
        return  self.user_id
    def get_time (self):
        return self.time



from apiclient import discovery
from SETMOK_API.entities.Mention import  Mention
from SETMOK_API.entities.User import User
import moment
class GooglePlus:

    def __init__(self, API_KEY):
        self.GPLUS = discovery.build('plus', 'v1', developerKey=API_KEY)


    def get_googleplus_data(self, keyword, limit) :
        mentionList=[]

        activity = self.GPLUS.activities().search(query=keyword, maxResults=20).execute()
        counter = 0
        while (True):


            nextToken = activity['nextPageToken']
            items = activity.get('items', [])
            for data in items:
                post = ' '.join(data['title'].strip().split())
                if post:
                    mention = Mention()
                    user=User()
                    #  print(TMPL % (data['actor']['displayName'],
                    #               data['published'], post) )
                    print(data)
                    print("----------------Post Info-----------------------")
                    mention.set_source('GooglePlus')

                    print("Post url: " + data["url"])
                    url=data["url"].split("/")
                    mention.set_status_id(url[-1])

                    print("Resharer Count: " + str(data["object"]["resharers"]['totalItems']))
                    mention.set_reshare_count(str(data["object"]["resharers"]['totalItems']))
                    print("Replies Count: " + str(data["object"]["replies"]['totalItems']))
                    print("Plusoners Count: " + str(data["object"]["plusoners"]['totalItems']))
                    print("Text: " + data["object"]['content'])
                    mention.set_text(data["object"]['content'])
                    for s in data['object']:
                        if s == 'attachments':
                            attachments = data["object"]['attachments']
                            for attachment in attachments:
                                print("attachments:" + attachment["url"])
                    print("----------------User Info-----------------------")
                    user.set_user_id(data['actor']["id"])
                    print("Diplay Name:" + data['actor']['displayName'])
                    user.set_display_name(data['actor']['displayName'])
                    user.set_display_picture( data['actor']['image']['url'])
                    print("Display Image:" + data['actor']['image']['url'])
                    print("Created at: " + data["published"])
                    m = moment.date(data["published"], '%Y-%m-%dT%H:%M:%SZ')
                    sql_format=m.format('YYYY-M-D H:mm:ss')
                    print(sql_format)
                    user.set_time(sql_format)
                    mention.set_user(user)
                    mentionList.append(mention)
                    counter = counter + 1
                    print("--------------------------------------------"+ str(counter))
                    if counter>=limit:
                        break
            if counter<limit :
             activity = self.GPLUS.activities().search(query=keyword, maxResults=20, pageToken=nextToken).execute()
            else:
                break
        return mentionList


class Mention:
    def __init__(self):
        self.text = None
        self.time = None
        self.reshare_count = None
        self.status_id=None
        self.source=None
        self.user=None
        self.resharers=None



    def set_text(self, text):
        self.text = text
    def set_time(self, time):
        self.time = time
    def set_reshare_count(self, reshare_count):
        self.reshare_count = reshare_count
    def set_status_id(self, status_id):
        self.status_id = status_id
    def set_source(self, source):
        self.source = source
    def set_user(self,user):
        self.user=user
    def set_resharer(self, resharers):
        self.resharers = resharers



    def get_text(self):
        return self.text
    def get_time(self):
        return self.time
    def get_reshare_count(self):
        return self.reshare_count
    def get_status_id(self):
        return self.status_id
    def get_source(self):
        return self.source
    def get_resharer(self):
        return self.resharers
    def get_user(self):
        return self.user




class Retweeter:
    def __init__(self):
        self.display_name = None
        self.display_picture = None
        self.time = None




    def set_display_name(self, display_name):
        self.display_name = display_name

    def set_display_picture(self, display_picture):
        self.display_picture = display_picture

    def set_time(self, time):
        self.time = time



    def get_display_name(self):
        return self.display_name

    def get_display_picture(self):
        return self.display_picture

    def get_time(self):
        return self.time



import  tweepy, json
from twitter import *
from SETMOK_API.entities.Mention import Mention
from SETMOK_API.entities.Retweeter import Retweeter
from SETMOK_API.entities.User import User
from datetime import datetime
import moment
class S_Twitter :

    def __init__(self, consumer_key, consumer_secret, oauth_token, oauth_token_secret):

     auth1 = tweepy.OAuthHandler(consumer_key, consumer_secret)
     auth1.set_access_token(oauth_token, oauth_token_secret)
     self.tweeter_api = tweepy.API(auth1)

     self.twitter = Twitter(auth = OAuth('2744706848-k2qvSjn2Z2RJuwuNi6nG3pUFfkV6LaQrjlPiX5C',
                               'Mg2rgYE8kqPmLDmWLEdVUka9yHybp3r4lY8Y1oEWatJ2n',
                               'cYIVpMAP9NhcEUURwYOmANLnU',
                  'qjmdfdoJpEaKAMgfjJ1WXnDRGzRzParxwfkCUh1WFvCxfRSBIu'))
    def get_data(self, tweets_list, mentionlist):

        for tweet in tweets_list:
            if(self.count<self.limit) :
                mention=Mention()
                user=User()
                # print("----------------Post Info-----------------------")
                mention.set_source("Twitter")
                mention.set_reshare_count(str(tweet["retweet_count"]))
                mention.set_text(tweet['text'])
                # # print(json.dumps(tweet, sort_keys=True, indent=4))
                mention.set_status_id(tweet["id_str"])
                print("Tweet ID: " + tweet["id_str"])
                # print("Created at: " + tweet["created_at"])

                sql_format = datetime.strptime(tweet["created_at"], '%a %b %d %H:%M:%S %z %Y')
                user.set_time(sql_format.strftime('%Y-%m-%d %H:%M:%S'))
                # t=datetime.strptime(tweet["created_at"], '%a %b %d %H:%M:%S %z %Y')
                # t.strftime('%Y-%m-%d %H:%M:%S')
                # m = moment.date(tweet["created_at"], '%a %b %d %H:%M:%S %z %Y')
                # sql_format = m.format('YYYY-M-D H:mm:ss')
                # print(sql_format)


                # print("Tweet Language: " + tweet["lang"])


                if tweet["retweet_count"] != 0:
                    user.set_display_name(tweet["retweeted_status"]["user"]["screen_name"])
                    user.set_display_picture(tweet["retweeted_status"]['user']['profile_image_url'])

                    mention.set_status_id(str(tweet["retweeted_status"]['id']))
                    # print("----------------Retweeted User Info-----------------------")
                    user.set_user_id(tweet["retweeted_status"]["user"]["id_str"])
                    # print("User Name: " + tweet["retweeted_status"]["user"]["name"])
                    # print("User Location: " + tweet["retweeted_status"]["user"]["location"])
                    user.set_total_post( str(tweet["retweeted_status"]['user']['statuses_count']))
                    user.set_following_count(str(tweet["retweeted_status"]['user']['friends_count']))
                    user.set_follower_count( str(tweet["retweeted_status"]['user']['followers_count']))
                    user.set_total_likes(str(tweet["retweeted_status"]['user']['favourites_count']))
                    # print("fav: " + str(tweet["retweeted_status"]['favorite_count']))


                    retweets = self.twitter.statuses.retweets._id(_id=str(tweet["retweeted_status"]['id']))
                    retweeterlist = []
                    for retweet in retweets:
                        retweeter = User()
                        retweeter.set_display_name(retweet["user"]["screen_name"])

                        #dt_str = datetime.datetime.strftime(retweet["created_at"], '%Y-%m-%d %H:%M:%S')

                        retweeter.set_time(retweet["created_at"])
                        retweeter.set_display_picture(retweet["user"]["profile_image_url"])

                        retweeter.set_user_id(retweet["user"]["id_str"])
                        retweeter.set_total_post(str(retweet['user']['statuses_count']))
                        retweeter.set_following_count(str(retweet['user']['friends_count']))
                        retweeter.set_follower_count(str(retweet['user']['followers_count']))
                        retweeter.set_total_likes(str(retweet['user']['favourites_count']))
                        retweeterlist.append(retweeter)
                        # print(" - retweeted by %s" % (retweet["user"]["screen_name"]))
                        # print("retweet time:"  +retweet["created_at"] )
                    mention.set_resharer(retweeterlist)
                else:
                    # print("----------------User Info-----------------------")
                    user.set_user_id(tweet["user"]["id_str"])
                    # print("User Name: " + tweet["user"]["name"])
                    # print("User Location: " + tweet["user"]["location"])

                    user.set_display_picture(tweet['user']['profile_image_url'])
                    user.set_display_name(tweet['user']['screen_name'])
                    user.set_total_post(str(tweet['user']['statuses_count']))
                    user.set_following_count( str(tweet['user']['friends_count']))
                    user.set_follower_count( str(tweet['user']['followers_count']))
                    user.set_total_likes(str(tweet['user']['favourites_count']))

                mention.set_user(user)
                mentionlist.append(mention)
                self.count=self.count+1
                print("<---------------------"+str(self.count)+"---------------------------->")
            else:
                    return mentionlist
        return mentionlist

    def get_twitter_data(self, keyword, limit):
      self.count=0
      self.limit=limit
      count=0
      mentionList=[]
      search_hashtag = tweepy.Cursor(self.tweeter_api.search, q=keyword).items(50)
      tweets_list = []
      for tweet in search_hashtag:
          tweets_list.append(json.loads(json.dumps(tweet._json)))
      id_ = tweets_list[len(tweets_list) - 1]["id"]
      mentionList=self.get_data(tweets_list ,mentionList)
      while (True):
        if self.count < self.limit :
          search_hashtag = tweepy.Cursor(self.tweeter_api.search, q=keyword, since_id=id_).items(50)
          for tweet in search_hashtag:
              print(json.dumps(tweet._json))
              for tweet in search_hashtag:
                  tweets_list.append(json.loads(json.dumps(tweet._json)))
              id_ = tweets_list[len(tweets_list) - 1]["id"]
              mentionList=self.get_data(tweets_list, mentionList)
              count=count+1
              print(count)

        else:
            break

      return mentionList


import setuptools

setuptools.setup(
    name="SET_MOKE",
    version="0.4",
    author="rehab shahzadi",
    author_email="rehab.shahzadi@kics.edu.pk",
    description="An API for social media data extraction",
    url="https://github.com/rehabshahzadi/SETMOK_API",
    packages=setuptools.find_packages(),

    classifiers = (
                  "Programming Language :: Python :: 3",
                  "Operating System :: OS Independent",

              ),
     install_requires=[
        'google-api-python-client',
        'tweepy==3.6.0',
        'twitter==1.18.0',
         'moment',
         'pymysql'
    ],

)



import configparser

class ConfigurationParser:
    parser = configparser.ConfigParser()

    def __init__(self, path):

        self.parser.read(path)
        self.secret_twitter_api_info = self.confParser("twitter_secrete_api_keys")
        self.secret_googleplus_api_info=self.confParser("googleplus_secrete_api_keys")


    def confParser(self, section):

        if not self.parser.has_section(section):
            print("No section information are available in config file for", section)
            return
        # Build dict
        tmp_dict = {}
        for option, value in self.parser.items(section):
            option = str(option)
            value = value.encode("utf-8")
            tmp_dict[option] = value
        return tmp_dict

    def get(self, src):

        if src == "googlePlus_credentials":
            return self.secret_googleplus_api_info
        elif src == "twitter_credentials":
            return self.secret_twitter_api_info